const Cost = require("./Cost");
const FlexRules = require("./FlexRules");
const Frequency = require("./Frequency");
const MenuAnalytics = require("./MenuAnalytics");
const { getWeek } = require('../Utils/DateUtil');
const ServiceConfig = require('./ServiceConfig');

class MenuReview {
  execute(menu, { restComb, calendar, utilizationGroupWeek, prato_combinacao, conjuntoPorcao }) {
    MenuAnalytics.data = menu.analytics;
    Frequency.load(menu.frequencias);
    menu.comBuraco = 0;
    menu.tabela.log.flexRules = {};
    Cost.initializeCost();
    menu.tabela.pratos.forEach((dishesOnDay, dayNumber) => {
      if(dayNumber === 0) {
        return;
      }
      /*rotina para recalcular percentuais de consumo dos pratos com base em atratividade ou historico*/
      let pesosAcum = {};
      let histAcum = {};
      const curDay = dayNumber.toString().padStart(2, '0');
      const menuStructure = menu.cardapio.datas.find(menuOnDay => menuOnDay.dia && menuOnDay.dia.startsWith(curDay));
      if(!menuStructure){
        let pu_acum_ant = menu.tabela.pudia_puacu[dayNumber - 1][1];
        menu.tabela.pudia_puacu[dayNumber] = [0, pu_acum_ant, 0, Cost.getTotalConsumers(), 0, Cost.getTotalCost()];
        return;
      }
      const { comensais, comenDIV } = menuStructure;
      let hasEmptyColumn = false;
      let sortDishes = false;
      dishesOnDay.forEach(dish => {
        if(!dish.prato && dish.pratosBKP && dish.pratosBKP.length && !dish.buracoObrigatorio) {
          const { dish: selectedDish, rules } = FlexRules.analyzeMenu(
            dish.data,
            dish.sequencial,
            {
              restComb,
              otherDishes: dish.pratosBKP,
              todayMenu: dishesOnDay,
              calendar,
              utilizationGroupWeek,
            }
          );
          if(selectedDish){
            const dishIdx = dish.pratosBKP.findIndex(curDish => curDish.prato === selectedDish.prato);
            dish.pratosBKP.splice(dishIdx, 1);
            const otherOptions = dish.pratosBKP.length ? [1] : [];
            Object.assign(dish, JSON.parse(JSON.stringify(selectedDish)), { ignoredRules: rules, otherOptions });
            menuStructure.pratos.push(JSON.parse(JSON.stringify(dish)));

            dish.tipoPrato = dish.COD_TIP_PRATO;
            dish.unidadePrato = dish.prato && dish.prato[0] == 'P' ? 'P' : menu.cardapio.unidade;
            dish.codprato = dish.prato;
            dish.prato = dish.unidadePrato == 'P' ? (dish.prato || '').substr(1) : (dish.prato || '').replace(menu.cardapio.unidade, '');
            dish.nrPratoCardapio = dish.sequencial;

            sortDishes = true;
            Frequency.addFrequency(dish, getWeek(dish.data));
            if (rules && rules.length) {
              menu.tabela.log.flexRules[dish.data] = menu.tabela.log.flexRules[dish.data] || {};
              menu.tabela.log.flexRules[dish.data][dish.sequencial] = rules;
            }
          } else {
            hasEmptyColumn = true;
          }
        }
        /*atualiza o acumulo de pesos das colunas atratividade + historico*/
        const { tipoPercConsumo } = ServiceConfig.getConfig(dish.sequencial);
        if (tipoPercConsumo && tipoPercConsumo ==  'ATRATIVIDADE' && dish.COD_GRP_TIP_PRATO && dish.VRPESOATRATIVIDADE) {
          pesosAcum[dish.COD_GRP_TIP_PRATO] = pesosAcum[dish.COD_GRP_TIP_PRATO]
            ? (pesosAcum[dish.COD_GRP_TIP_PRATO] += parseInt(dish.VRPESOATRATIVIDADE))
            : parseInt(dish.VRPESOATRATIVIDADE);
        } else {
          if (tipoPercConsumo && dish.COD_GRP_TIP_PRATO && dish.prato) {
            histAcum[dish.COD_GRP_TIP_PRATO] = histAcum[dish.COD_GRP_TIP_PRATO]
              ? histAcum[dish.COD_GRP_TIP_PRATO] + parseFloat(dish.percentualConsumo)
              : parseFloat(dish.percentualConsumo);
          }
        }        
      });
      /*percorre novamente todos os sequenciais do dia, recalculando o percentual de consumo de cada prato de acordo com o historico ou atratividade*/
      dishesOnDay.forEach(dish => {
        const { tipoPercConsumo } = ServiceConfig.getConfig(dish.sequencial);
        let percAT = conjuntoPorcao.find((cp) => cp.CDGRTIPOPR == dish.COD_GRP_TIP_PRATO);
        let pesPrato = pesosAcum[dish.COD_GRP_TIP_PRATO];
        let isATV = (tipoPercConsumo && tipoPercConsumo == 'ATRATIVIDADE' && dish.prato && dish.COD_GRP_TIP_PRATO &&  dish.VRPESOATRATIVIDADE != null)

        if (pesPrato && dish.VRPESOATRATIVIDADE && isATV != null) {
          let percHist = histAcum[dish.COD_GRP_TIP_PRATO] || 0;
          if (percAT) {
            if (percAT.VRCONJUNTOPORCAO > percHist) {
              percAT = percAT.VRCONJUNTOPORCAO - percHist;
            } else {
              percAT = percAT.VRCONJUNTOPORCAO;
            }
          } else {
            percAT = 100;
          }
          dish.percentualConsumo = (dish.VRPESOATRATIVIDADE / pesPrato) * percAT;
        }
        /*se houver uma combinacao que reduz o percentual de consumo de um prato, realiza o calculo*/
        let isCombReduced = prato_combinacao.find(
          (pc) =>
            ((pc.COD_GRP_TIP_PRATO_COMB == dish.COD_GRP_TIP_PRATO && !pc.NRGRUPOPRATOSERV_COMB && !pc.COD_PRATO_COMB) ||
              pc.NRGRUPOPRATOSERV_COMB == dish.gpprato ||
              pc.COD_PRATO_COMB == dish.prato) &&
              dishesOnDay.find((cd) => pc.COD_PRATO == cd.prato || pc.NRGRUPOPRATOSERV == cd.gpprato),
        );
        if (isCombReduced && isCombReduced.PERCENTREDUCAO && isCombReduced.PERCENTREDUCAO != 0) {
          dish.percentualConsumo =
            parseFloat(dish.percentualConsumo) -
            (parseFloat(dish.percentualConsumo) * parseFloat(isCombReduced.PERCENTREDUCAO)) / 100;
        }        
        /*por fim recalcula o pu de cada prato de acordo com seu percentual de consumo recalculado*/
        if(dish.prato) {
          Cost.calculate(dish, { comensais, comenDIV });
        }
      });      
      if(hasEmptyColumn) {
        menu.comBuraco++;
      }
      if(sortDishes) {
        menuStructure.pratos.sort((dishA, dishB) => Number(dishA.sequencial) < Number(dishB.sequencial) ? -1 : 1);
      }
      Cost.totalizeDay(comenDIV);
      menuStructure.mediaCusto = Cost.costAvg;
      menu.tabela.pudia_puacu[dayNumber] = [Cost.dayPu, Cost.getTotalPu(), comenDIV, Cost.getTotalConsumers(), Cost.dayPu  * parseInt((comenDIV || 0), 10) , Cost.getTotalCost()];
      Cost.nextDay();
    });
    menu.pu_aculm = menu.tabela.pudia_puacu[menu.tabela.pudia_puacu.length - 1][1];
    menu.cust_total = menu.tabela.pudia_puacu[menu.tabela.pudia_puacu.length - 1][5];
    menu.dif_custo = menu.custo_pdr - menu.pu_aculm;
  }
}

module.exports = new MenuReview();
