const timeout = require('connect-timeout');
const express = require('express');
const cors = require('cors');
const fs = require('fs');

const cluster = require('cluster');
const maxInstances = require('os').cpus().length;

const CardController = require("./js/CardController");
const config = require("./config/config");
const port = config.port;
const app = express();
app.use(cors());

// process.stdout.write('\033c');
// console.log('Starting Server...');

var handleMainReq = (req, res) => {
    let body = '';
    req.on('data', chunk => {
        // convert Buffer to string
        body += (null || chunk).toString();
    });
    req.on('end', () => {
        try{
            let currentDate = new Date();
            const formattedDate = currentDate.toLocaleString('pt-BR');            
            const customHeader = req.headers['x-custom-header'];
            console.log(`\n[${formattedDate}]`);
            if(customHeader){
                console.log(`Requisição recebida de ${customHeader}`);
            }
            let data = JSON.parse(body);
            //fs.writeFileSync('./cardapio/apresentacao.json', body);
            let meta = {
                'meta': parseFloat(data.P_VRCUSTPADR),
                'metaMin': parseFloat(data.VLR_CUSTO_MIN),
                'metaMax': parseFloat(data.VLR_CUSTO_MAX),
            };
            let startTime = Date.now();
            if (data.configuracao_servico && data.configuracao_servico.data && data.configuracao_servico.data[0]) {
		console.log(`Unidade:${data.configuracao_servico.data[0].COD_FILIAL} Serviço:${data.configuracao_servico.data[0].COD_SERVICO} Mês:${data.P_DTMES} Operador:${data.CDOPERADOR}`);
                let { trie, perfectCard } = CardController.generatePerfectCard(data, meta);
                let diff = (Date.now() - startTime)/1000;
                console.log(`${trie} Cards generated [${perfectCard.trie}] - Elapsed time: [${diff}] Dias com buraco: ${perfectCard.comBuraco}`);
                res.status(200);
                res.json({
                    'sucesso': true,
                    'data': perfectCard
                });
                res.end();

            } else {
                res.status(500).send({
                    'sucesso': false,
                    'otmmessage': 'ALG_CONF_NOT_FOUND',
                    'title': 'ALG_PROB_WITH_REGISTRATION'
                });
            }
        } catch( error ){
            console.log(error);
            res.status(500).send({
                'sucesso': false,
                'otmmessage': error.message + '\n' + error.stack,
                'title': 'ERRO_EXECUCAO'
            });
            res.end();
        }
    });
};

if (cluster.isMaster) {
    console.log('Server Started...');
    console.log(`Listening on port ${port}`);

    // Fork workers.
    for (let i = 0; i < maxInstances; i++) {
        cluster.fork();
    }

    cluster.on('exit', (worker, code, signal) => {
        console.log(`worker ${worker.process.pid} died`);
    });
} else {
    app.post('/', handleMainReq);
    app.listen(port);
    app.use(timeout(9900000));
}
